/*****************************************************************************/
/*!	\file 			Pis10Client.cpp
 *	\brief                  PIS10 stack wrapper
 *	\par 			SystemCORP Embedded Technology Pty Ltd
 *
 *				15/50 William St.,
 *				Beckenham,
 *				Perth, WA 6107,
 *				Australia.
 *
 *				Phone	: +61 (8) 9258 4683
 *				Fax	: +61 (8) 9258 4673
 *				Email	: info@systemcorp.com.au
 */
/*****************************************************************************/

/******************************************************************************
*	Include
******************************************************************************/
#include <stdio.h>
#include <QtGui/QApplication>
#include <QtGui>
#include "QtClientWidget.h"
#include <time.h>
#include "pis10Client.h"

QtClientWidget * PIS10Client::MyQTwidget = NULL;
PIS10Client * PIS10Client::pThis = NULL;

/*****************************************************************************
* Constructor
******************************************************************************
* Constructor for PIS10 Object.
*
******************************************************************************/
PIS10Client::PIS10Client(QObject *parent) :
    QObject(parent)
{
    MyClient = NULL;
    pThis = this;
    /* here we create, loadSCL and start the stack
       Then create the QT object and show the gui
    */
}


/*****************************************************************************
* RunServer() Function
******************************************************************************
* Calls create server.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: RunClient()
{
    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;

    eError = CreateClient(IEC61850_OPTION_NONE);// should be IEC61850_OPTION_NONE
    if(eError == IEC61850_ERROR_NONE)
    {
        printf("   IEC61850 Client is Created \n");
        eError = LoadSCLFile("TqClient.cid");

        if(eError == IEC61850_ERROR_NONE)
        {
            printf("SCL File Loaded\n");
            eError = StartMyClient();
            if(eError == IEC61850_ERROR_NONE)
            {
                printf("Client Started \n");
            }
        }
    }
    return eError;
}

/*****************************************************************************
* RunGUI() Function
******************************************************************************
* Runs GUI.
*
******************************************************************************/
int PIS10Client :: RunGUI(int argc, char *argv[])
{
    QApplication a(argc, argv);

    MyQTwidget = new QtClientWidget();
    MyQTwidget->ptrToPIS10 = this;
    MyQTwidget->show();

    RunClient();

    return a.exec();
}

/*****************************************************************************
* CreateClient() Function
******************************************************************************
* This function sets all the IEC61850 parameters and then calls
* IEC61850_Create API
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: CreateClient(unsigned long int inOptions)
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE ;
    struct IEC61850_Parameters tClientParam ;

    memset(&tClientParam, 0, sizeof(struct IEC61850_Parameters) );


    tClientParam.ClientServerFlag    = IEC61850_CLIENT;	// This is a IEC 61850 Server
    tClientParam.uiOptions           = inOptions; // No options set
    tClientParam.ptReadCallback      = (IEC61850_ReadCallback) ReadCallbackHandler; // Assign Read Callback Function
    tClientParam.ptWriteCallback     = (IEC61850_WriteCallback) WriteCallbackHandler; // Assign Write Callback Function
    tClientParam.ptUpdateCallback    = (IEC61850_UpdateCallback) UpdateCallbackHandler; // SETS Callback Function for Server
    tClientParam.ptSelectCallback    = NULL; // SETS Select commands
    tClientParam.ptOperateCallback   = NULL; // SETS Operate commands
    tClientParam.ptCancelCallback    = NULL; // SETS Cancel commands
    //	tClientParam.uiMaxAssociations	 = 0;

    if(MyClient == NULL)
    {
        MyClient = IEC61850_Create(&tClientParam, &eErrorCode);

        if(MyClient == NULL)
        {
            printf("   Server Failed to create: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
        }
    }
    else
    {
        eErrorCode = IEC61850_ERROR_INVALID_STATE;
    }

    return eErrorCode;
}

/*****************************************************************************
* LoadSCLFile() Function
******************************************************************************
* This function takes the ICD file pointer and then calls
* IEC61850_LoadSCLFile
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: LoadSCLFile(char *pFileName)
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;
    IEC61850 myIEC61850Object = NULL;;

    if(pFileName != NULL)
    {
        myIEC61850Object = GetMyClient();

        eErrorCode = IEC61850_LoadSCLFile(myIEC61850Object, pFileName); // Load in IED configuration SCL file
        if(eErrorCode != IEC61850_ERROR_NONE)
        {
            printf("   Loading SCL file (%s) has failed: %s (%i)\n", pFileName, IEC61850_ErrorString(eErrorCode), eErrorCode);
        }
    }
    else
    {
        eErrorCode = IEC61850_ERROR_INVALID_PARAMETERS;
    }

    return eErrorCode;
}

/*****************************************************************************
* StartMyServer() Function
******************************************************************************
* This function starts IEC61850 Server by calling IEC61850_Start.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: StartMyClient()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Start(GetMyClient()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}

/*****************************************************************************
* StopMyServer() Function
******************************************************************************
* This function stops IEC61850 Server by calling IEC61850_Stop.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: StopMyClient()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Stop(GetMyClient()); // Starts myServer

    if(eErrorCode != IEC61850_ERROR_NONE)
    {
        printf("   Can't start server: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}

/*****************************************************************************
* FreeMyServer() Function
******************************************************************************
* This function frees IEC61850 Server by calling IEC61850_Free.
*
******************************************************************************/
enum IEC61850_ErrorCodes PIS10Client :: FreeMyClient()
{
    enum IEC61850_ErrorCodes eErrorCode = IEC61850_ERROR_NONE;

    eErrorCode = IEC61850_Free(GetMyClient());  //Frees myServerClient

    if(eErrorCode == IEC61850_ERROR_NONE)
    {
        MyClient = NULL;
    }
    else
    {
        printf("   Free IEC61850 Object has failed: %s (%i)\n", IEC61850_ErrorString(eErrorCode), eErrorCode);
    }

    return eErrorCode;
}

/*****************************************************************************
* GetMyServer() Function
******************************************************************************
* This function returns PIS10 server instance.
*
******************************************************************************/
IEC61850 PIS10Client :: GetMyClient()
{
    return MyClient;
}


/*****************************************************************************
* ReadCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to read an element
* on the server.
******************************************************************************/
//Read Callback to be called by the PIS-10 Stack
int PIS10Client :: ReadCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData *ptReturnedValue)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Read CallBack =======\n");
        printf("     The error code is: %i\n", iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptDataAttributeID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptReturnedValue);
        printf("\n");

        printf("\n     ==============================\n");
        return  iErrorCode;
}

/*****************************************************************************
* WriteCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to write an element
* on the server.
******************************************************************************/
//Write Callback to be called by the PIS-10 Stack
int PIS10Client :: WriteCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID *ptDataAttributeID, struct IEC61850_DataAttributeData * ptNewValue)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Write CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptDataAttributeID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptNewValue);
        printf("\n");

        printf("\n     ==============================\n");
        return (int) iErrorCode;
}

/*****************************************************************************
* SelectCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to select an control
* element on the server.
******************************************************************************/
//Select Callback to be called by the PIS-10 Stack
int  PIS10Client ::SelectCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptSelectValue, int iSyncroCheck, int iInterlockCheck)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Select CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptSelectValue);
        printf("\n");

        printf("\n     ==============================\n");
        return (int) iErrorCode;
}

/*****************************************************************************
* OperateCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to operate an control
* element on the server.
******************************************************************************/
//Operate Callback to be called by the PIS-10 Stack
int  PIS10Client ::OperateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, struct IEC61850_DataAttributeData * ptOperateValue, int iSyncroCheck, int iInterlockCheck)
{
        Integer32 iErrorCode = IEC61850_COMMAND_ERROR_NONE; //Create return value and init to No Error

        printf("\n\n     ========= Operate CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     The DAData is: ");
        IEC61850_PrintDataAttributeData(ptOperateValue);
        printf("\n");

        printf("\n     ==============================\n");




        return (int) iErrorCode;
}

/*****************************************************************************
* CancelCallbackHandler() Function
******************************************************************************
* This function gets callbacked when a client tries to cancel selection
*  of an control element on the server.
******************************************************************************/
//Cancel Callback to be called by the PIS-10 Stack
int  PIS10Client ::CancelCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID)
{
        Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error

        printf("\n     ========= Cancel CallBack =======\n");
        printf("     The error code is: %i\n",(int) iErrorCode);
        printf("     The DAID is: ");
        IEC61850_PrintDataAttributeID(ptControlID);
        printf("\n     NO DAData for Cancel");
        printf("\n");
        printf("\n     ==============================\n");

        return (int) iErrorCode;
}

/*****************************************************************************
* UpdateCallbackHandler() Function
******************************************************************************
* This function is not used in server side
*
******************************************************************************/
//Update Callback to be called by the PIS-10 Stack
void  PIS10Client ::UpdateCallbackHandler( void *ptUserData, struct IEC61850_DataAttributeID * ptControlID, const struct IEC61850_DataAttributeData *ptNewValue)
{
    Integer32 iErrorCode = IEC61850_CB_ERROR_NONE; //Create return value and init to No Error
    Unsigned8 u8Value = 0;

    enum IEC61850_ErrorCodes eError = IEC61850_ERROR_NONE ;

    IEC61850_DataAttributeID_Generic * DAID = (IEC61850_DataAttributeID_Generic *) ptControlID;
    struct IEC61850_DataAttributeData *ptUpDtDAData = (struct IEC61850_DataAttributeData *)ptNewValue;

    printf("\n     ========= Update CallBack =======\n");
    printf("     The error code is: %i\n",(int) iErrorCode);
    printf("     The DAID is: ");
    IEC61850_PrintDataAttributeID(ptControlID);
    printf("\n     The DAData is: ");
    IEC61850_PrintDataAttributeData(ptNewValue);
    printf("\n");
    printf("\n     ==============================\n");



    if(DAID->uiField1 == 1 && DAID->uiField3 == 1 && DAID->uiField4 == 0 && DAID->uiField5 == 0)
    {
        MyQTwidget->LedController(DAID->uiField2, *(Boolean*)ptUpDtDAData->pvData);
    }
    else if((DAID->uiField1 == 4 || DAID->uiField1 == 5)  && DAID->uiField3 == 2 && DAID->uiField4 == 0 && DAID->uiField5 == 0)
    {
        //printf("Update=> DAID->uiField1: %d, cValue: %d\n",DAID->uiField1,*(unsigned char*)ptUpDtDAData->pvData );
        MyQTwidget->XCBR_CSWIController(DAID->uiField1,*(unsigned char*)ptUpDtDAData->pvData);
    }
}
